@@ -666,8 +666,9 @@ pref_right_margin (HTMLPainter *p, HTMLClueFlow *clueflow, HTMLObject *o, gint y
 	  
 	if (clueflow->style == HTML_CLUEFLOW_STYLE_PRE || ! HTML_IS_PLAIN_PAINTER(p))
 		return fixed_margin;
-	
-	return MIN (fixed_margin, 72 * html_painter_get_space_width (p, GTK_HTML_FONT_STYLE_SIZE_3, NULL));
+
+	return MIN (fixed_margin, 72 * (MAX (html_painter_get_space_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL),
+					     html_painter_get_e_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL))));
 }
 
 static void
@@ -832,7 +833,8 @@ layout_line (HTMLObject *o, HTMLPainter *painter, HTMLObject *begin,
 	cur = begin;
 
 	old_y = o->y;
-	html_object_calc_size (begin, painter, changed_objs);
+	if (!HTML_IS_TEXT_SLAVE (begin) || HTML_IS_TEXT (begin->prev))
+		html_object_calc_size (begin, painter, changed_objs);
 
 	a = d = height = 0;
 	update_height (begin, html_object_get_valign (begin), &a, &d, &height, &top_align);
@@ -1085,7 +1087,20 @@ get_roman_value (gint value, gboolean lower)
 static gchar *
 get_item_marker_str (HTMLClueFlow *flow, gboolean ascii_only)
 {
-	switch (flow->item_type) {
+	HTMLListType type = flow->item_type;
+
+	if (type == HTML_LIST_TYPE_BLOCKQUOTE && flow->levels->len > 0) {
+		int i;
+
+		for (i = flow->levels->len - 1; i >= 0; i --) {
+			if (flow->levels->data [i] != HTML_LIST_TYPE_BLOCKQUOTE) {
+				type = flow->levels->data [i];
+				break;
+			}
+		}
+	}
+
+	switch (type) {
 	case HTML_LIST_TYPE_ORDERED_ARABIC:
 		return g_strdup_printf ("%d. ", flow->item_number);
 	case HTML_LIST_TYPE_ORDERED_LOWER_ALPHA:
