@@ -210,6 +210,7 @@ copy (HTMLObject *self,
 	dimg->alt = g_strdup (simg->alt);
 	dimg->usemap = g_strdup (simg->usemap);
 	dimg->final_url = NULL;
+	dimg->animation_active = FALSE;
 
 	/* add dest to image_ptr interests */
 	dimg->image_ptr->interests = g_slist_prepend (dimg->image_ptr->interests, dimg);
@@ -304,7 +305,7 @@ calc_preferred_width (HTMLObject *o,
 }
 
 static gboolean
-calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
+html_image_real_calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 {
 	HTMLImage *image;
 	guint pixel_size;
@@ -592,7 +593,7 @@ save (HTMLObject *self,
 		return FALSE;	
 
 	if (image->percent_width) {
-		if (!html_engine_save_output_string (state, " WIDTH=\"%d\%\"", image->specified_width))
+		if (!html_engine_save_output_string (state, " WIDTH=\"%d%%\"", image->specified_width))
 			return FALSE;
 	} else if (image->specified_width > 0) {
 		if (!html_engine_save_output_string (state, " WIDTH=\"%d\"", image->specified_width))
@@ -600,7 +601,7 @@ save (HTMLObject *self,
 	}
 
 	if (image->percent_height) {
-		if (!html_engine_save_output_string (state, " HEIGHT=\"%d\%\"", image->specified_height))
+		if (!html_engine_save_output_string (state, " HEIGHT=\"%d%%\"", image->specified_height))
 			return FALSE;
 	} else if (image->specified_height > 0) {
 		if (!html_engine_save_output_string (state, " HEIGHT=\"%d\"", image->specified_height))
@@ -768,7 +769,7 @@ html_image_class_init (HTMLImageClass *image_class,
 	object_class->destroy = destroy;
 	object_class->calc_min_width = calc_min_width;
 	object_class->calc_preferred_width = calc_preferred_width;
-	object_class->calc_size = calc_size;
+	object_class->calc_size = html_image_real_calc_size;
 	object_class->check_point = check_point;
 	object_class->get_url = get_url;
 	object_class->get_target = get_target;
@@ -805,6 +806,7 @@ html_image_init (HTMLImage *image,
 
 	html_object_init (object, HTML_OBJECT_CLASS (klass));
 
+	image->animation_active = FALSE;
 	image->url = g_strdup (url);
 	image->target = g_strdup (target);
 	image->usemap = NULL;
@@ -1102,7 +1104,7 @@ html_image_factory_end_pixbuf (GtkHTMLStream *stream,
 
 	update_or_redraw (ip);
 	if (ip->factory->engine->opened_streams && ip->factory->engine->block_images)
-		ip->factory->engine->opened_streams --;
+		html_engine_opened_streams_decrement (ip->factory->engine);
 	/* printf ("IMAGE(%p) opened streams: %d\n", ip->factory->engine, ip->factory->engine->opened_streams); */
 	if (ip->factory->engine->opened_streams == 0 && ip->factory->engine->block && ip->factory->engine->block_images)
 		html_engine_schedule_update (ip->factory->engine);
@@ -1151,7 +1153,9 @@ html_image_pointer_update (HTMLImagePointer *ip)
 	for (cur = ip->interests; cur; cur = cur->next) {
 		HTMLImage           *image = cur->data;
 		
-		if (image && image->animation_active) {
+		if (image && image->animation_active && html_object_is_parent (engine->clue, HTML_OBJECT (image))) {
+			DA (printf ("queue draw (%p)\n", image);)
+
 			image->animation_active = FALSE;
 			html_engine_queue_draw (engine, HTML_OBJECT (image));
 		}
@@ -1381,7 +1385,7 @@ html_image_pointer_load (HTMLImagePointer *ip)
 	html_image_pointer_ref (ip);
 
 	if (ip->factory->engine->block_images)
-		ip->factory->engine->opened_streams ++;
+		html_engine_opened_streams_increment (ip->factory->engine);
 	return gtk_html_stream_new (GTK_HTML (ip->factory->engine->widget),
 				    html_image_factory_types,
 				    html_image_factory_write_pixbuf,
@@ -1563,14 +1567,16 @@ html_image_factory_deactivate_animations (HTMLImageFactory *factory)
 static void
 ref_image_ptr (gpointer key, gpointer val, gpointer data)
 {
-	html_image_pointer_ref (HTML_IMAGE_POINTER (val));
+	if (HTML_IMAGE_POINTER (val)->animation)
+		html_image_pointer_ref (HTML_IMAGE_POINTER (val));
 	/* printf ("ref(%p) %s --> %d\n", val, HTML_IMAGE_POINTER (val)->url, HTML_IMAGE_POINTER (val)->refcount); */
 }
 
 static void
 unref_image_ptr (gpointer key, gpointer val, gpointer data)
 {
-	html_image_pointer_unref (HTML_IMAGE_POINTER (val));
+	if (HTML_IMAGE_POINTER (val)->animation)
+		html_image_pointer_unref (HTML_IMAGE_POINTER (val));
 }
 
 void
