@@ -94,9 +94,10 @@ copy (HTMLObject *self,
 	dest->percent = self->percent;
 	dest->flags = self->flags;
 	dest->redraw_pending = FALSE;
-	dest->selected = self->selected;
+	dest->selected = FALSE;
 	dest->free_pending = FALSE;
 	dest->change = self->change;
+	dest->draw_focused = FALSE;
 
 	g_datalist_init (&dest->object_data);
 	html_object_copy_data_from_object (dest, self);
@@ -132,7 +133,10 @@ op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GL
 		} else
 			html_object_move_cursor_before_remove (self, e);
 
-		html_object_change_set   (self,  HTML_CHANGE_ALL_CALC);
+		html_object_change_set (self, HTML_CHANGE_ALL_CALC);
+		html_object_change_set (self->parent, HTML_CHANGE_ALL_CALC);
+		/* force parent redraw */
+		self->parent->width = 0;
 		html_object_remove_child (self->parent, self);
 		*len += html_object_get_recursive_length (self);
 
@@ -144,6 +148,11 @@ op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GL
 static gboolean
 merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList **left, GList **right, HTMLCursor *cursor)
 {
+	if (self->parent) {
+		html_object_change_set (self->parent, HTML_CHANGE_ALL_CALC);
+		self->parent->width = 0;
+	}
+
 	return FALSE;
 }
 
@@ -213,7 +222,7 @@ fit_line (HTMLObject *o,
 }
 
 static gboolean
-calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
+html_object_real_calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 {
 	return FALSE;
 }
@@ -605,7 +614,7 @@ html_object_class_init (HTMLObjectClass *klass,
 	klass->draw = draw;
 	klass->is_transparent = is_transparent;
 	klass->fit_line = fit_line;
-	klass->calc_size = calc_size;
+	klass->calc_size = html_object_real_calc_size;
 	klass->set_max_width = set_max_width;
 	klass->set_max_height = set_max_height;
 	klass->get_left_margin = get_left_margin;
