@@ -57,7 +57,7 @@ static void         spell_error_destroy     (SpellError *se);
 static void         move_spell_errors       (GList *spell_errors, guint offset, gint delta);
 static GList *      remove_spell_errors     (GList *spell_errors, guint offset, guint len);
 static void         remove_text_slaves      (HTMLObject *self);
-static void         html_text_change_attrs  (PangoAttrList *attr_list, GtkHTMLFontStyle style, HTMLEngine *e, gint start_index, gint end_index);
+static void         html_text_change_attrs  (PangoAttrList *attr_list, GtkHTMLFontStyle style, HTMLEngine *e, gint start_index, gint end_index, gboolean avoid_default_size);
 
 /* void
 debug_spell_errors (GList *se)
@@ -923,7 +923,7 @@ html_text_calc_text_size (HTMLText *t, HTMLPainter *painter,
 }
 
 gint
-html_text_calc_part_width (HTMLText *text, HTMLPainter *painter, gint offset, gint len, gint *asc, gint *dsc)
+html_text_calc_part_width (HTMLText *text, HTMLPainter *painter, char *start, gint offset, gint len, gint *asc, gint *dsc)
 {
 	gint idx, width = 0, line_offset;
 
@@ -940,16 +940,23 @@ html_text_calc_part_width (HTMLText *text, HTMLPainter *painter, gint offset, gi
 
 	line_offset = html_text_get_line_offset (text, painter, offset);
 
+	if (start == NULL)
+		start = html_text_get_text (text, offset);
+
 	if (HTML_IS_GDK_PAINTER (painter) || HTML_IS_PLAIN_PAINTER (painter)) {
 		HTMLTextPangoInfo *pi;
-		gchar *s = html_text_get_text (text, offset);
+		PangoLanguage *language = NULL;
+		PangoFont *font = NULL;
+		gchar *s = start;
 
 		pi = html_text_get_pango_info (text, painter);
 
 		idx = html_text_get_item_index (text, painter, offset, &offset);
-		if (asc || dsc)
+		if (asc || dsc) {
 			update_asc_dsc (painter, pi->entries [idx].item, asc, dsc);
-
+			font = pi->entries [idx].item->analysis.font;
+			language = pi->entries [idx].item->analysis.language;
+		}
 		while (len > 0) {
 			if (*s == '\t') {
 				gint skip = 8 - (line_offset % 8);
@@ -963,15 +970,16 @@ html_text_calc_part_width (HTMLText *text, HTMLPainter *painter, gint offset, gi
 			if (offset >= pi->entries [idx].item->num_chars - 1) {
 				idx ++;
 				offset = 0;
-				if (len > 0 && (asc || dsc))
+				if (len > 0 && (asc || dsc) && (pi->entries [idx].item->analysis.font != font || pi->entries [idx].item->analysis.language != language)) {
 					update_asc_dsc (painter, pi->entries [idx].item, asc, dsc);
+				}
 			} else
 				offset ++;
 			s = g_utf8_next_char (s);
 		}
 		width = PANGO_PIXELS (width);
 	} else {
-		html_text_calc_text_size (text, painter, html_text_get_text (text, offset), len, NULL, NULL, 0, &line_offset,
+		html_text_calc_text_size (text, painter, start, len, NULL, NULL, 0, &line_offset,
 					  html_text_get_font_style (text), text->face, &width, asc, dsc);
 	}
 
@@ -987,7 +995,7 @@ calc_preferred_width (HTMLObject *self,
 
 	text = HTML_TEXT (self);
 
-	width = html_text_calc_part_width (text, painter, 0, text->text_len, &self->ascent, &self->descent);
+	width = html_text_calc_part_width (text, painter, text->text, 0, text->text_len, &self->ascent, &self->descent);
 	if (html_clueflow_tabs (HTML_CLUEFLOW (self->parent), painter)) {
 		gint line_offset;
 		gint tabs;
@@ -1110,6 +1118,21 @@ html_text_pango_info_get_index (HTMLTextPangoInfo *pi, gint byte_offset, gint id
 }
 
 
+static void
+html_text_add_cite_color (PangoAttrList *attrs, HTMLText *text, HTMLClueFlow *flow, HTMLEngine *e)
+{
+	HTMLColor *cite_color = html_colorset_get_color (e->settings->color_set, HTMLCiteColor);
+
+	if (cite_color && flow->levels->len > 0 && flow->levels->data[0] == HTML_LIST_TYPE_BLOCKQUOTE_CITE) {
+		PangoAttribute *attr;
+
+		attr = pango_attr_foreground_new (cite_color->color.red, cite_color->color.green, cite_color->color.blue);
+		attr->start_index = 0;
+		attr->end_index = text->text_bytes;
+		pango_attr_list_change (attrs, attr);
+	}
+}
+
 HTMLTextPangoInfo *
 html_text_get_pango_info (HTMLText *text, HTMLPainter *painter)
 {
@@ -1124,6 +1147,8 @@ html_text_get_pango_info (HTMLText *text, HTMLPainter *painter)
 		GList *items, *cur;
 		PangoAttrList *attrs;
 		PangoAttribute *attr;
+		HTMLClueFlow *flow = NULL;
+		HTMLEngine *e = NULL;
 		gchar *translated, *heap = NULL;
 		gint i;
 
@@ -1133,8 +1158,18 @@ html_text_get_pango_info (HTMLText *text, HTMLPainter *painter)
 			translated = alloca (text->text_bytes);
 
 		html_replace_tabs (text->text, translated, text->text_bytes);
+		attrs = pango_attr_list_new ();
+
+		if (HTML_OBJECT (text)->parent && HTML_IS_CLUEFLOW (HTML_OBJECT (text)->parent))
+			flow = HTML_CLUEFLOW (HTML_OBJECT (text)->parent);
+		
+		if (painter->widget && GTK_IS_HTML (painter->widget))
+			e = GTK_HTML (painter->widget)->engine;
+
+		if (flow && e)
+			html_text_add_cite_color (attrs, text, flow, e);
+
 		if (HTML_IS_PLAIN_PAINTER (painter)) {
-			attrs = pango_attr_list_new ();
 			attr = pango_attr_family_new (painter->font_manager.fixed.face);
 			attr->start_index = 0;
 			attr->end_index = text->text_bytes;
@@ -1146,38 +1181,36 @@ html_text_get_pango_info (HTMLText *text, HTMLPainter *painter)
 				pango_attr_list_insert (attrs, attr);
 			}
 		} else
-			attrs = pango_attr_list_copy (text->attr_list);
+			pango_attr_list_splice (attrs, text->attr_list, 0, 0);
+
 		if (text->extra_attr_list)
-			pango_attr_list_splice (attrs, text->extra_attr_list, 0, text->text_len);
-		if (HTML_OBJECT (text)->parent && HTML_IS_CLUEFLOW (HTML_OBJECT (text)->parent) && painter->widget && GTK_IS_HTML (painter->widget)) {
-			GtkHTMLFontStyle parent_style = html_clueflow_get_default_font_style (HTML_CLUEFLOW (HTML_OBJECT (text)->parent));
+			pango_attr_list_splice (attrs, text->extra_attr_list, 0, 0);
+		if (!HTML_IS_PLAIN_PAINTER (painter)) {
+			if (flow && e)
+				html_text_change_attrs (attrs, html_clueflow_get_default_font_style (flow), GTK_HTML (painter->widget)->engine, 0, text->text_bytes, TRUE);
 
-			html_text_change_attrs (attrs, parent_style, GTK_HTML (painter->widget)->engine, 0, text->text_bytes);
-		}
-		if (text->links && painter->widget && GTK_IS_HTML (painter->widget)) {
-			PangoAttrList *links_attrs = pango_attr_list_new ();
-			HTMLColor *link_color = html_colorset_get_color (GTK_HTML (painter->widget)->engine->settings->color_set, HTMLLinkColor);
-			GSList *l;
+			if (text->links && e) {
+				HTMLColor *link_color = html_colorset_get_color (e->settings->color_set, HTMLLinkColor);
+				GSList *l;
 
-			for (l = text->links; l; l = l->next) {
-				PangoAttribute *attr;
-				Link *link;
+				for (l = text->links; l; l = l->next) {
+					PangoAttribute *attr;
+					Link *link;
 
-				link = (Link *) l->data;
-				attr = pango_attr_underline_new (PANGO_UNDERLINE_SINGLE);
-				attr->start_index = link->start_index;
-				attr->end_index = link->end_index;
-				pango_attr_list_insert_before (links_attrs, attr);
-
-				attr = pango_attr_foreground_new (link_color->color.red, link_color->color.green, link_color->color.blue);
-				attr->start_index = link->start_index;
-				attr->end_index = link->end_index;
-				pango_attr_list_change (links_attrs, attr);
+					link = (Link *) l->data;
+					attr = pango_attr_underline_new (PANGO_UNDERLINE_SINGLE);
+					attr->start_index = link->start_index;
+					attr->end_index = link->end_index;
+					pango_attr_list_change (attrs, attr);
 
+					attr = pango_attr_foreground_new (link_color->color.red, link_color->color.green, link_color->color.blue);
+					attr->start_index = link->start_index;
+					attr->end_index = link->end_index;
+					pango_attr_list_change (attrs, attr);
+				}
 			}
-			pango_attr_list_splice (attrs, links_attrs, 0, text->text_len);
-			pango_attr_list_unref (links_attrs);
 		}
+
 		if (text->select_length) {
 			gchar *end;
 			gchar *start;
@@ -1502,28 +1535,8 @@ save_text_part (HTMLText *text, HTMLEngineSaveState *state, guint start_index, g
 }
 
 static gboolean
-save_link_open (Link *link, HTMLEngineSaveState *state, GSList **attrs)
+save_link_open (Link *link, HTMLEngineSaveState *state)
 {
-	GSList *l, *next;
-
-	if (attrs)
-		for (l = *attrs; l; l = next) {
-			PangoAttribute *attr = (PangoAttribute *) l->data;
-			next = l->next;
-			if (attr->klass->type == PANGO_ATTR_UNDERLINE) {
-				*attrs = g_slist_delete_link (*attrs, l);
-				pango_attribute_destroy (attr);
-			} else if (attr->klass->type == PANGO_ATTR_FOREGROUND) {
-				GdkColor *color = &html_colorset_get_color (state->engine->settings->color_set, HTMLLinkColor)->color;
-				PangoAttrColor *ac = (PangoAttrColor *) attr;
-
-				if (ac->color.red == color->red && ac->color.green == color->green && ac->color.blue == color->blue) {
-					*attrs = g_slist_delete_link (*attrs, l);
-					pango_attribute_destroy (attr);
-				}
-			}
-		}
-
 	return html_engine_save_output_string (state, "<A HREF=\"%s\">", link->url);
 }
 
@@ -1534,22 +1547,31 @@ save_link_close (Link *link, HTMLEngineSaveState *state)
 }
 
 static gboolean
-save_text (HTMLText *text, HTMLEngineSaveState *state, guint start_index, guint end_index, GSList **l, gboolean link_started)
+save_text (HTMLText *text, HTMLEngineSaveState *state, guint start_index, guint end_index, GSList **l, gboolean *link_started)
 {
-	if (link_started && *l) {
+	if (*l) {
 		Link *link;
 
 		link = (Link *) (*l)->data;
 
-		while (*l && link->end_index < end_index) {
-			if (!save_text_part (text, state, start_index, link->end_index))
-				return FALSE;
-			save_link_close (link, state);
-			(*l) = (*l)->next;
-			start_index = link->end_index;
-			if (*l) {
-				link = (Link *) (*l)->data;
-				save_link_open (link, state, NULL);
+		while (*l && ((!*link_started && start_index <= link->start_index && link->start_index < end_index)
+			      || (*link_started && link->end_index <= end_index))) {
+			if (!*link_started && start_index <= link->start_index && link->start_index < end_index) {
+				if (!save_text_part (text, state, start_index, link->start_index))
+					return FALSE;
+				*link_started = TRUE;
+				save_link_open (link, state);
+				start_index = link->start_index;
+			}
+			if (*link_started && link->end_index <= end_index) {
+				if (!save_text_part (text, state, start_index, link->end_index))
+					return FALSE;
+				save_link_close (link, state);
+				*link_started = FALSE;
+				(*l) = (*l)->next;
+				start_index = link->end_index;
+				if (*l)
+					link = (Link *) (*l)->data;
 			}
 		}
 
@@ -1584,31 +1606,14 @@ save (HTMLObject *self, HTMLEngineSaveState *state)
 			if (end_index > text->text_bytes)
 				end_index = text->text_bytes;
 
-			if (l && !link_started) {
-				Link *link = (Link *) l->data;
-
-				if (link && link->start_index == start_index) {
-					save_link_open (link, state, &attrs);
-					link_started = TRUE;
-				}
-			}
 			if (attrs)
 				save_open_attrs (state, attrs);
-			save_text (text, state, start_index, end_index, &l, link_started);
+			save_text (text, state, start_index, end_index, &l, &link_started);
 			if (attrs) {
 				attrs = g_slist_reverse (attrs);
 				save_close_attrs (state, attrs);
 				html_text_free_attrs (attrs);
 			}
-			if (l && link_started) {
-				Link *link = (Link *) l->data;
-
-				if (link->end_index == end_index) {
-					save_link_close (link, state);
-					l = l->next;
-					link_started = FALSE;
-				}
-			}
 		} while (pango_attr_iterator_next (iter));
 		g_slist_free (links);
 	}
@@ -2203,7 +2208,8 @@ get_cursor_base (HTMLObject *self,
 		    || HTML_OBJECT_TYPE (obj->next) != HTML_TYPE_TEXTSLAVE) {
 			html_object_calc_abs_position (obj, x, y);
 			if (offset > slave->posStart)
-				*x += html_text_calc_part_width (HTML_TEXT (self), painter, slave->posStart, offset - slave->posStart, NULL, NULL);
+				*x += html_text_calc_part_width (HTML_TEXT (self), painter, html_text_slave_get_text (slave),
+								 slave->posStart, offset - slave->posStart, NULL, NULL);
 
 			return;
 		}
@@ -2790,12 +2796,12 @@ html_text_get_link_rectangle (HTMLText *text, HTMLPainter *painter, gint offset,
 		gint xs, ys, xe, ye;
 
 		html_object_calc_abs_position (HTML_OBJECT (start), &xs, &ys);
-		xs += html_text_calc_part_width (text, painter, start->posStart, link->start_offset - start->posStart, NULL, NULL);
+		xs += html_text_calc_part_width (text, painter, html_text_slave_get_text (start), start->posStart, link->start_offset - start->posStart, NULL, NULL);
 		ys -= HTML_OBJECT (start)->ascent;
 
 		html_object_calc_abs_position (HTML_OBJECT (end), &xe, &ye);
 		xe += HTML_OBJECT (end)->width;
-		xe -= html_text_calc_part_width (text, painter, link->end_offset, end->posStart + start->posLen - link->end_offset, NULL, NULL);
+		xe -= html_text_calc_part_width (text, painter, text->text + link->end_index, link->end_offset, end->posStart + start->posLen - link->end_offset, NULL, NULL);
 		ye += HTML_OBJECT (end)->descent;
 
 		*x1 = MIN (xs, xe);
@@ -3006,6 +3012,13 @@ calc_font_size_filter (PangoAttribute *attr, gpointer data)
 
 	if (attr->klass->type == PANGO_ATTR_SIZE)
 		html_pango_attr_font_size_calc ((HTMLPangoAttrFontSize *) attr, e);
+	else if (attr->klass->type == PANGO_ATTR_FAMILY) {
+		/* FIXME: this is not very nice. we set it here as it's only used when fonts changed.
+		   once family in style is used again, that code must be updated */
+		PangoAttrString *sa = (PangoAttrString *) attr;
+		g_free (sa->value);
+		sa->value = g_strdup (e->painter->font_manager.fixed.face);
+	}
 
 	return FALSE;
 }
@@ -3097,7 +3110,7 @@ html_text_get_style_conflicts (HTMLText *text, GtkHTMLFontStyle style, gint star
 }
 
 static void
-html_text_change_attrs (PangoAttrList *attr_list, GtkHTMLFontStyle style, HTMLEngine *e, gint start_index, gint end_index)
+html_text_change_attrs (PangoAttrList *attr_list, GtkHTMLFontStyle style, HTMLEngine *e, gint start_index, gint end_index, gboolean avoid_default_size)
 {
 	PangoAttribute *attr;
 
@@ -3137,18 +3150,21 @@ html_text_change_attrs (PangoAttrList *attr_list, GtkHTMLFontStyle style, HTMLEn
 		pango_attr_list_change (attr_list, attr);
 	}
 
-	/* always apply size so that we can zoom */
-	attr = html_pango_attr_font_size_new (style);
-	html_pango_attr_font_size_calc ((HTMLPangoAttrFontSize *) attr, e);
-	attr->start_index = start_index;
-	attr->end_index = end_index;
-	pango_attr_list_change (attr_list, attr);
+	if (!avoid_default_size
+	    || (((style & GTK_HTML_FONT_STYLE_SIZE_MASK) != GTK_HTML_FONT_STYLE_DEFAULT)
+		&& ((style & GTK_HTML_FONT_STYLE_SIZE_MASK) != GTK_HTML_FONT_STYLE_SIZE_3))) {
+		attr = html_pango_attr_font_size_new (style);
+		html_pango_attr_font_size_calc ((HTMLPangoAttrFontSize *) attr, e);
+		attr->start_index = start_index;
+		attr->end_index = end_index;
+		pango_attr_list_change (attr_list, attr);
+	}
 }
 
 void
 html_text_set_style_in_range (HTMLText *text, GtkHTMLFontStyle style, HTMLEngine *e, gint start_index, gint end_index)
 {
-	html_text_change_attrs (text->attr_list, style, e, start_index, end_index);
+	html_text_change_attrs (text->attr_list, style, e, start_index, end_index, FALSE);
 }
 
 void
@@ -3231,16 +3247,6 @@ html_text_get_first_color_in_range (HTMLText *text, HTMLEngine *e, gint start_in
 			pango_attr_iterator_range (iter, &iter_start_index, &iter_end_index);
 			if (MAX (iter_start_index, start_index) <= MIN (iter_end_index, end_index)) {
 				color = color_from_attrs (iter);
-				if (html_color_equal (color, html_colorset_get_color (e->settings->color_set, HTMLLinkColor))) {
-					gint offset = g_utf8_pointer_to_offset (text->text, text->text + MAX (start_index, iter_start_index));
-
-					if (html_text_get_link_at_offset (text, offset)) {
-						html_color_unref (color);
-						color = NULL;
-						/* color = html_colorset_get_color (e->settings->color_set, HTMLLinkColor);
-						   html_color_ref (color); */
-					}
-				}
 				break;
 			}
 		} while (pango_attr_iterator_next (iter));
@@ -3248,10 +3254,10 @@ html_text_get_first_color_in_range (HTMLText *text, HTMLEngine *e, gint start_in
 		pango_attr_iterator_destroy (iter);
 	}
 
-	/* if (!color) {
+	if (!color) {
 		color = html_colorset_get_color (e->settings->color_set, HTMLTextColor);
 		html_color_ref (color);
-		} */
+	}
 
 	return color;
 }
